import React, { Component } from "react";
import axios from "axios";
import { Redirect, withRouter } from "react-router-dom";
import PropTypes from "prop-types";

import InputComponent from "../common";

import { Consumer } from "../../index";

// const Aux = props => props.children

// <Fragment></Fragment>

// <> </>

class SignIn extends Component {
	static propTypes = {
		age: PropTypes.number,
		string: PropTypes.string
	};

	state = {
		singIn: {
			email: {
				value: "",
				config: {
					type: "email",
					name: "email",
					placeholder: "Enter your email"
				},
				valid: false,
				touch: false,
				validation: {
					required: true
				}
			},
			password: {
				value: "",
				config: {
					type: "password",
					name: "password",
					placeholder: "Enter your password"
				},
				valid: false,
				touch: false,
				validation: {
					required: true,
					minLength: 5
				}
			}
		},
		isValid: false,
		token: null
	};

	submit = func => e => {
		e.preventDefault();
		const { singIn } = this.state;
		// https://test-app-a-level.herokuapp.com
		const values = Object.keys(singIn).reduce((prev, elem) => ({ ...prev, [elem]: singIn[elem].value }), {});
		console.log("values", values);
		// 1
		axios({
			url: "https://test-app-a-level.herokuapp.com/auth/login",
			method: "POST",
			headers: {
				"Content-Type": "application/json"
			},
			data: values
		})
			.then(res => func(res.data.token)) // 2
			.catch(err => console.log(err.response)); // 3
	};

	validator = (value, rules) => {
		let isValid = true;

		if (rules.required) {
			isValid = value.trim() !== "" && isValid;
		}

		if (rules.minLength) {
			isValid = value.length >= rules.minLength && isValid;
		}

		return isValid;
	};

	change = e => {
		const { name, value } = e.target;

		this.setState(prevState => {
			const valid = this.validator(value, prevState.singIn[name].validation);

			const otherValid = Object.keys(prevState.singIn).some(el => !prevState.singIn[el].valid);

			return {
				singIn: {
					...prevState.singIn,
					[name]: {
						...prevState.singIn[name],
						value,
						touch: true,
						valid
					}
				},
				isValid: otherValid && valid
			};
		});
	};

	render() {
		const { singIn, isValid } = this.state;
		// console.log("this.", this.props);

		return (
			<Consumer>
				{({ token, setToken }) =>
					token ? (
						<Redirect to="/" />
					) : (
						<form className="auth-box__sign-in-form" onSubmit={this.submit(setToken)}>
							{Object.keys(singIn).map(el => (
								<InputComponent
									key={el}
									touch={singIn[el].touch}
									valid={!singIn[el].valid}
									config={{ ...singIn[el].config, value: singIn[el].value, onChange: this.change }}
								/>
							))}
							<button type="submit">Sign in</button>
						</form>
					)
				}
			</Consumer>
		);
	}
}

export default withRouter(SignIn);
